math.step = function(current, target, step)
	step = step or 1
	if current < target then
		return math.min(current + step, target)
	else
		return math.max(target, current - step)
	end
end

math.sign = function(value)
	if value > 0 then
		return 1
	elseif value < 0 then
		return -1
	else
		return 0
	end
end

math.lerp = function(a, b, t)
	return a + (b - a) * t
end

math.clamp = function(a, min, max)
	if min > max then
		min, max = max, min
	end

	if a >= min and a <= max then
		return a -- most popular case, first
	elseif a < min then
		return min
	else
		return max
	end
end

math.round = function(num, numDecimalPlaces)
	local mult = 10^(numDecimalPlaces or 0)
	return math.floor(num * mult + 0.5) / mult
end

math.trunc = function(num)
	if num>0 then
		return math.floor(num)
	else
		return math.ceil(num)
	end
end

math.is = function(value, ...)
	for i = 1, select("#", ...) do
		if value == select(i, ...) then
			return true
		end
	end
	return false
end

math.equal = function(a, b, eps)
	eps = eps or 0.001
	return math.abs(a - b) < eps
end

math.length = function(x1, y1, x2, y2)
	return math.sqrt((x2-x1)^2 + (y2-y1)^2)
end

math.chance = function(percent)
	return math.random(1, 100) <= (percent * 100)
end

--- return number +- number*percent_delta
math.get_random_around = function(number, percent_delta)
	return number + number * (-percent_delta + percent_delta*2*math.random()) -- [-percent_delta, percent_delta]
end

table.get_item = function(array, key, value)
	for i = 1, #array do
		if array[i][key] == value then
			return array[i]
		end
	end
	return nil
end

table.contains = function(t, value)
	for i = 1, #t do
		if t[i] == value then
			return i
		end
	end
	return false
end

table.length = function(array)
	local length = 0
	if array then
		for key, value in pairs(array) do
			length = length + 1
		end
	end
	return length
end

table.clear = function(array)
	for i = #array, 1, -1 do
		array[i] = nil
	end
end

table.remove_item = function(t, value)
	local index = table.contains(t, value)
	if index then
		table.remove(t, index)
	end
end

table.remove_by_dict = function(t, example)
	local index = -1
	for i = 1, #t do
		local v = t[i]
		local is_match = true
		for key, val in pairs(example) do
			if v[key] ~= val then
				is_match = false
			end
		end
		if is_match then
			index = i
			break
		end
	end
	if index ~= -1 then
		table.remove(t, index)
	end
end

table.is_empty = function(t)
	if not t or type(t) ~= 'table' then
		return true
	end
	return next(t) == nil
end

table.extend = function(t1, t2)
	for k, v in pairs(t2) do
		t1[k] = v
	end
end

table.convert_number_to_string = function(t)
	local r = {}
	for k, v in pairs(t) do
		if type(v) == "table" then
			v = table.convert_number_to_string(v)
		end
		r[tostring(k)] = v
	end
	return r
end

table.convert_string_to_number = function(t)
	local ret = {}
	for k, v in pairs(t) do
		if type(v) == "table" then
			v = table.convert_string_to_number(v)
		end
		local tmp_key = tonumber(k)
		if tmp_key ~= nil then
			ret[tmp_key] = v
		else
			ret[k] = v
		end
	end
	return ret
end

table.weight_random = function(weight_table, count)
	count = count or 1
	local items = {}
	for i = 1, count do
		local w = 0
		for k, v in pairs(weight_table) do
			if not table.contains(items, k) then
				w = w + v
			end
		end
		if w == 0 then
			break
		end
		w = w * math.random()
		local last
		for k, v in pairs(weight_table) do
			if not table.contains(items, k) then
				last = k
				w = w - v
				if w <= 0 then
					table.insert(items, k)
					last = nil
					break
				end
			end
		end
		if last then
			table.insert(items, last)
		end
	end
	return items
end

table.deepcopy = function(orig)
	local copy
	if type(orig) == 'table' then
		copy = {}
		for orig_key, orig_value in next, orig, nil do
			copy[table.deepcopy(orig_key)] = table.deepcopy(orig_value)
		end
		setmetatable(copy, table.deepcopy(getmetatable(orig)))
	else
		copy = orig
	end
	return copy
end

table.erase = function(t)
	for k, v in pairs(t) do
		v = nil
		t[k] = nil
	end
end

table.random_list_item = function(list)
	local chance = math.random(1, table.length(list))
	local counter = 1
	for k, v in pairs(list) do
		if counter == chance then
			return k, v
		end
		counter = counter + 1
	end
end
